#include "ui.h"

using namespace ge211;

using column_t = Connect4_model::column_t;

static Position board_to_screen_(Position board_pos)
{
    int x = 2 * token_radius * (Connect4_model::n - board_pos.y - 1);
    int y = 2 * token_radius * board_pos.x;
    return {x, y};
}

static Position screen_to_board_(Position screen_pos)
{
    int col_no = screen_pos.x / (2 * token_radius);
    int row_no = Connect4_model::n - screen_pos.y / (2 * token_radius) - 1;
    return {col_no, row_no};
}

void Connect4_ui::draw(ge211::Sprite_set& sprites)
{
    // Here we add a sprite for each token in the game:
    for (int col_no = 0; col_no < Connect4_model::m; ++col_no) {
        const column_t* column = model_.get_column(col_no);
        for (int row_no = 0; row_no < column->size(); ++row_no) {
            Player player = (*column)[row_no];
            Position screen_pos = board_to_screen_({col_no, row_no});
            sprites.add_sprite(select_sprite_(player), screen_pos);
        }
    }

    // Here we possibly add a sprite as a cursor that follows the mouse:
    if (model_.is_good_col(mouse_column_)) {
        int col_no             = mouse_column_;
        const column_t* column = model_.get_column(col_no);
        int row_no             = int(column->size());
        Player player          = model_.get_turn();
        if (row_no < Connect4_model::n && player != Player::neither) {
            Position screen_pos = board_to_screen_({col_no, row_no});
            sprites.add_sprite(select_sprite_(player), screen_pos, true);
        }
    }
}

void Connect4_ui::on_mouse_move(Position screen_pos)
{
    screen_to_board_(screen_pos).x;
}

Dimensions Connect4_ui::initial_window_dimensions() const
{
    return {2 * token_radius * Connect4_model::m,
            2 * token_radius * Connect4_model::n};
}

Circle_sprite const&
Connect4_ui::select_sprite_(Player player, bool ghost) const
{
    if (player == Player::first)
        return ghost? player1_token_ : ghost1_token_;
    else
        return ghost? player2_token_ : ghost2_token_;
}

std::string Connect4_ui::initial_window_title() const
{
    return "Connect Four";
}

void Connect4_ui::on_mouse_down(Mouse_button btn, Position screen_posn)
{
    if (model_.get_turn() == Player::neither || btn != Mouse_button::left)
        return;

    int col_no = screen_to_board_(screen_posn);

    if (model_.is_playable_col(col_no))
        model_.place_token(col_no);
}


